"""
@author: tyrantlucifer
@contact: tyrantlucifer@gmail.com
@blog: https://tyrantlucifer.com
@file: factory.py
@time: 2022/6/12 00:15
@desc: uploader工厂
"""

from typora_upload.upload_utils import *


class UploaderFactory(object):

    def __init__(self):
        pass

    @staticmethod
    def get_uploader(storage):
        if storage == 'oss':
            return OssUploader()
        elif storage == 'github':
            return GithubUploader()



"""
@author: tyrantlucifer
@contact: tyrantlucifer@gmail.com
@blog: https://tyrantlucifer.com
@file: upload_utils.py
@time: 2022/6/11 18:57
@desc:
"""

import json
import base64
import oss2
import re
import requests

from typora_upload.config_utils import *


class Common(object):

    def __init__(self):
        pass

    @staticmethod
    def is_http_url(path):
        if re.match(r'(http|https)://([\w.]+/?)\S*', path):
            return True
        else:
            return False

    @staticmethod
    def generate_file_name(file):
        postfix = file.split(".")[-1]
        prefix = time.strftime('%Y%m%d%H%M%S')
        filename = prefix + '.' + postfix
        return filename

    @staticmethod
    def get_http_file_content(path):
        result = requests.get(path)
        return result.content

    @staticmethod
    def get_local_file_content(path):
        with open(path, "rb") as file:
            content = file.read()
        return content


class OssUploader(object):

    def __init__(self):
        self.access_key_id = ConfigUtils.get_value('oss', 'access_key_id')
        self.access_key_secret = ConfigUtils.get_value('oss', 'access_key_secret')
        self.bucket_name = ConfigUtils.get_value('oss', 'bucket_name')
        self.endpoint = ConfigUtils.get_value('oss', 'endpoint')
        self.path_prefix = ConfigUtils.get_value('oss', 'path_prefix')
        self.domain_name = ConfigUtils.get_value('oss', 'domain_name')
        self.bucket = oss2.Bucket(oss2.Auth(self.access_key_id, self.access_key_secret), self.endpoint,
                                  self.bucket_name)

    def upload(self, path):
        if Common.is_http_url(path):
            filename = path.split("/")[-1]
            filename = Common.generate_file_name(filename)
            content = Common.get_http_file_content(path)
            self.bucket.put_object('{0}/{1}'.format(self.path_prefix, filename), content)
        else:
            if sys.platform.find("win") >= 0:
                filename = path.split("\\")[-1]
            else:
                filename = path.split("/")[-1]
            filename = Common.generate_file_name(filename)
            self.bucket.put_object_from_file('{0}/{1}'.format(self.path_prefix, filename), path)
        return 'https://{0}/{1}/{2}'.format(self.domain_name, self.path_prefix, filename)


class GithubUploader(object):

    def __init__(self):
        self.user = ConfigUtils.get_value('github', 'user')
        self.repo = ConfigUtils.get_value('github', 'repo')
        self.path_prefix = ConfigUtils.get_value('github', 'path_prefix')
        self.token = ConfigUtils.get_value('github', 'token')
        self.api_url = "https://api.github.com/repos/{0}/{1}/contents/{2}/".format(
            self.user,
            self.repo,
            self.path_prefix)
        self.prefix_url = "https://cdn.jsdelivr.net/gh/{0}/{1}/{2}/".format(
            self.user,
            self.repo,
            self.path_prefix
        )
        self.headers = {
            "Authorization": "token " + self.token,
            "Accept": "application/vnd.github.v3+json",
            "Connection": 'close'
        }

    def upload(self, path):
        if Common.is_http_url(path):
            filename = path.split("/")[-1]
            filename = Common.generate_file_name(filename)
            content = Common.get_http_file_content(path)
        else:
            if sys.platform.find("win") >= 0:
                filename = path.split("\\")[-1]
            else:
                filename = path.split("/")[-1]
            filename = Common.generate_file_name(filename)
            content = Common.get_local_file_content(path)
        content = base64.b64encode(content)
        content = str(content, 'utf-8')
        url = self.__generate_api_url(filename)
        commit = self.__generate_commit_message(filename)
        data = {
            "message": commit,
            "content": content
        }
        data = json.dumps(data)
        result = requests.put(url, headers=self.headers, data=data)
        result.close()
        return os.path.join(self.prefix_url, filename)

    def __generate_api_url(self, filename):
        return os.path.join(self.api_url, filename)

    def __generate_commit_message(self, filename):
        current_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
        return current_time + " " + "COMMIT" + " " + filename



"""
@author: tyrantlucifer
@contact: tyrantlucifer@gmail.com
@blog: https://tyrantlucifer.com
@file: functions.py
@time: 2022/6/11 19:35
@desc:
"""

from typora_upload.factory import *


class Upload(object):

    def __init__(self):
        pass

    @staticmethod
    def upload(path):
        storage = ConfigUtils.get_value('default', 'storage')
        uploader = UploaderFactory.get_uploader(storage)
        image_url = uploader.upload(path)
        print("Upload Success:")
        print(image_url)


class InitStorgeConfig(object):

    def __init__(self):
        pass

    @staticmethod
    def init_oss():
        access_key_id = input("access_key_id:")
        ConfigUtils.set_value('oss', "access_key_id", access_key_id)
        access_key_secret = input("access_key_secret:")
        ConfigUtils.set_value('oss', "access_key_secret", access_key_secret)
        bucket_name = input("bucket_name:")
        ConfigUtils.set_value('oss', "bucket_name", bucket_name)
        endpoint = input("endpoint:")
        ConfigUtils.set_value('oss', "endpoint", endpoint)
        path_prefix = input("path_prefix:")
        ConfigUtils.set_value('oss', "path_prefix", path_prefix)
        domain_name = input("domain_name:")
        ConfigUtils.set_value('oss', "domain_name", domain_name)

    @staticmethod
    def init_github():
        user = input("user:")
        ConfigUtils.set_value('github', 'user', user)
        repo = input("repo:")
        ConfigUtils.set_value('github', 'repo', repo)
        path_prefix = input("path_prefix:")
        ConfigUtils.set_value('github', 'path_prefix', path_prefix)
        token = input("token:")
        ConfigUtils.set_value('github', 'token', token)

    @staticmethod
    def update_storage(storage):
        ConfigUtils.set_value('default', 'storage',storage)


"""
@author: tyrantlucifer
@contact: tyrantlucifer@gmail.com
@blog: https://tyrantlucifer.com
@file: logger.py
@time: 2021/2/18 19:59
@desc: 初始化全局log变量
"""

import logging
import time

from typora_upload.init_utils import *
from typora_upload.print_utils import *

# 初始化工具类init_utils.InitConfig
init_config = InitConfig()

# 初始化打印工具类
color = Colored()

# 初始化全局logger记录格式及级别
logger = logging.getLogger("typora-upload")
logger.setLevel(logging.INFO)

# 初始化全局logger记录格式
formatter = logging.Formatter('%(asctime)s - %(filename)s[line:%(lineno)d] - [%(funcName)s] - %(levelname)s: %('
                              'message)s')

# 初始化全局logger文件记录格式及级别
log_file_handler = logging.FileHandler(init_config.log_file)
log_file_handler.setLevel(logging.INFO)
log_file_handler.setFormatter(formatter)
logger.addHandler(log_file_handler)


# 定义计算函数运行时间装饰器
def calculate(func):
    def main(*args, **kwargs):
        start = time.time()
        func(*args, **kwargs)
        logger.info("Func - {0} Total time: {1}s".format(func.__name__,
                                                         round(time.time() - start, 2)))
    return main



"""
@author: tyrantlucifer
@contact: tyrantlucifer@gmail.com
@blog: https://tyrantlucifer.com
@file: main.py
@time: 2021/2/18 21:36
@desc: typora-upload入口函数
"""

import argparse
from typora_upload.functions import *


def get_parser():
    parser = argparse.ArgumentParser(description=color.blue("The typora image upload plugin based Python."),
                                     epilog=color.yellow('Powered by ') + color.green('tyrantlucifer') + color.yellow(
                                         ". If you have any questions,you can send e-mails to ") + color.green(
                                         "tyrantlucifer@gmail.com"))
    parser.add_argument("-u", "--upload", metavar="file_path", help="upload image file")
    parser.add_argument("-s", "--storage", metavar="storage", help="storage type")
    parser.add_argument("-i", "--init", metavar="storage", help="init storage config")
    parser.add_argument("-v", "--version", action="store_true", help="display version")
    return parser


def main():
    parser = get_parser()
    args = parser.parse_args()
    if args.upload:
        Upload.upload(args.upload)
    elif args.init:
        if args.init == 'oss':
            InitStorgeConfig.init_oss()
        elif args.init == 'github':
            InitStorgeConfig.init_github()
    elif args.storage:
        InitStorgeConfig.update_storage(args.storage)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



"""
@author: tyrantlucifer
@contact: tyrantlucifer@gmail.com
@blog: https://tyrantlucifer.com
@file: print_utils.py
@time: 2021/2/18 19:28
@desc: 提供打印功能工具类
"""

import qrcode
from colorama import init, Fore


class Colored(object):
    """生成不同颜色字体工具类

    通过传入颜色，返回对应终端的颜色格式化字体

    属性:
        color: 支持颜色集合
    """

    def __init__(self):
        init(autoreset=False)
        self.color = (
            'red',
            'green',
            'yellow',
            'white',
            'blue'
        )

    @staticmethod
    def red(s):
        return Fore.LIGHTRED_EX + s + Fore.RESET

    @staticmethod
    def green(s):
        return Fore.LIGHTGREEN_EX + s + Fore.RESET

    @staticmethod
    def yellow(s):
        return Fore.LIGHTYELLOW_EX + s + Fore.RESET

    @staticmethod
    def white(s):
        return Fore.LIGHTWHITE_EX + s + Fore.RESET

    @staticmethod
    def blue(s):
        return Fore.LIGHTBLUE_EX + s + Fore.RESET

    def print(self, text, color):
        if color == 'red':
            print(self.red(text))
        if color == 'green':
            print(self.green(text))
        if color == 'yellow':
            print(self.yellow(text))
        if color == 'white':
            print(self.white(text))
        if color == 'blue':
            print(self.blue(text))


class PrintQrcode(object):
    """打印二维码工具类

    可使用该工具类的静态方法通过字符串打印任意的二维码

    属性:
        None
    """

    def __init__(self):
        pass

    @staticmethod
    def print_qrcode(string):
        qr = qrcode.QRCode(
            version=1,
            error_correction=qrcode.constants.ERROR_CORRECT_L,
            box_size=3,
            border=1,
        )
        qr.add_data(string)
        qr.make(fit=True)
        qr.print_ascii(tty=True, invert=True)



"""
@author: tyrantlucifer
@contact: tyrantlucifer@gmail.com
@blog: https://tyrantlucifer.com
@file: init_utils.py
@time: 2021/2/18 10:31
@desc: 初始化工具类集合
"""

import os
import sys
import platform
import configparser


class InitConfig(object):
    """初始化工具类

    当typora-upload启动时初始化相关属性

    属性:
        version: 版本号
        platform: 操作系统平台
        system: 操作系统
        home_dir: 家目录路径
        config_dir: 配置目录路径
        config_file: 配置文件
        config_lock_file: 配置锁文件
        log_file: typora-upload日志记录文件
    """

    def __init__(self):
        self.version = '1.0.0'
        self.platform = sys.platform
        self.system = platform.uname().version
        self.home_dir = os.path.expanduser('~')
        self.config_dir = os.path.join(self.home_dir,
                                       '.typora-upload')
        self.config_file = os.path.join(self.config_dir,
                                        'config.ini')
        self.config_lock_file = os.path.join(self.config_dir,
                                             '.config.lock')
        self.log_file = os.path.join(self.config_dir,
                                     'typora-upload.log')
        if not self.__is_exist_config_dir():
            self.__create_config_dir()

    def __is_exist_config_dir(self):
        if os.path.exists(self.config_dir):
            return True
        else:
            return False

    def __init_config_file(self):
        config = configparser.ConfigParser()
        config.add_section('default')
        config.set('default', 'storage', 'oss')
        config.add_section('github')
        config.add_section('oss')
        with open(self.config_file, 'w+') as file:
            config.write(file)
        with open(self.config_lock_file, 'w') as lock_file:
            lock_file.write('')

    def __create_config_dir(self):
        os.mkdir(self.config_dir)
        with open(self.config_file, 'w') as file:
            file.write('')
        self.__init_config_file()



import codecs
from setuptools import setup

with codecs.open('README.rst', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="typora-upload",
    version="1.0.0",
    author="tyrantlucifer",
    author_email="tyrantlucifer@gmail.com",
    description="The typora image upload plugin based Python.",
    url="https://github.com/tyrantlucifer/typora-upload",
    packages=[
        "typora_upload"
    ],
    entry_points={
        'console_scripts': [
            'typora-upload = typora_upload.main:main'
        ]
    },
    install_requires=[
        "requests",
        "qrcode",
        "colorama",
        "oss2"
    ],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: Proxy Servers',
    ],
    long_description=long_description,
)



"""
@author: tyrantlucifer
@contact: tyrantlucifer@gmail.com
@blog: https://tyrantlucifer.com
@file: config_utils.py
@time: 2021/2/18 22:42
@desc:
"""

from typora_upload.logger import *


class ConfigUtils(object):
    """配置项工具类

    提供从本地配置文件中读取对应参数的功能

    属性:
        config: 配置文件对象
    """
    config = configparser.ConfigParser()
    config.read(init_config.config_file)

    def __init__(self):
        pass

    @staticmethod
    def get_value(section, key):
        return ConfigUtils.config.get(section, key)

    @staticmethod
    def set_value(section, key, value):
        ConfigUtils.config.set(section, key, str(value))
        with open(init_config.config_file, 'w+') as file:
            ConfigUtils.config.write(file)

